import logging
import os
import sys
import time
import asyncio

from pscr_common.log import setup_logging, clear_logging_handlers
from pyformance import timer
from dre_c_rule_updater.rms_rules_manager import RMSSegmentController
from dre_c_rule_updater.s3_manager import PSCS3Manager
from dre_c_rule_updater.s3_manager import RMSS3Manager
from dre_c_rule_updater.config import Config
from dre_c_rule_updater.app_metrics import Metrics

logging.getLogger("boto").setLevel(logging.CRITICAL)
logging.getLogger("botocore").setLevel(logging.CRITICAL)
logging.getLogger("s3transfer").setLevel(logging.CRITICAL)
logging.getLogger("urllib3").setLevel(logging.CRITICAL)

_logger = logging.getLogger(__name__)
config = Config.create()


def str_to_bool(value):
    if value.lower() == "true":
        return True
    if value.lower() == "false":
        return False
    raise AttributeError("Invalid boolean string passed as env var")


def main():
    clear_logging_handlers()
    if str_to_bool(os.environ.get("PSCR_LOGGING", "true")):
        setup_logging(
            level=config.get("log", "level", fallback="INFO"),
            svc_version=config.get("service", "version"),
            env_id=config.get("env", "id"),
            svc_id="TODO",
        )
    else:
        logging.basicConfig(level=config.get("log", "level", fallback="DEBUG"))

    metrics = Metrics(config)
    reporter = metrics.get_reporter()

    _logger.info("Dre-C Rule Updater Started")

    if str_to_bool(config.get("dre_c", "dry_run")):
        _logger.info("dre-c-rule-updater running in DRY RUN Mode")
        metrics.add_dry_run_metrics()
        # Making thread sleep to print logs in order
        time.sleep(0.1)
        _logger.info("Dre-C Rule Updater Finished")
        if reporter is not None:
            asyncio.get_event_loop().run_until_complete(reporter.report_now())
        sys.exit()

    validate_domain(config.get("dre_c", "host_name"))

    with timer("dre_c_rule_updater.rms.query.time").time():
        rms_segment_controller = RMSSegmentController(config, metrics, reporter)
        # Call Search Segment API to get the Segment Id
        segment_id = rms_segment_controller.get_drec_aggregate_segment()
        _logger.debug(f"dre-c-rule-updater SEGMENT ID {segment_id}")
        # Call Segment Data Search API to get the Segment S3 Url
        seg_data_search_response = (
            rms_segment_controller.get_drec_aggregate_segment_data(segment_id)
        )
    rms_s3_manager = RMSS3Manager(
        seg_data_search_response.get("url"), metrics, reporter
    )
    content_hash = seg_data_search_response.get("content_hash")
    created_at = str(seg_data_search_response.get("created_at"))
    _logger.info(f"Dre-C Rule Content-Hash  {content_hash}")
    psc_s3_manager = PSCS3Manager(config, metrics, reporter)
    # Validate Rule Set
    if psc_s3_manager.validate_current_rules(content_hash):
        _logger.info("No changes in Rules therefore not uploading")
        # Making thread sleep to print logs in order
        time.sleep(0.1)
        _logger.info("Dre-C Rule Updater Finished")

    else:
        # Download The Latest Rule Set
        with timer("dre_c_rule_updater.rule.download.time").time():
            rules = rms_s3_manager.download_drec_aggregate_rule_set()
        # Back Up Current Rule Set
        with timer("dre_c_rule_updater.rule.backup.time").time():
            psc_s3_manager.create_backup()
        # Upload Rules to PSC S3 Bucket
        with timer("dre_c_rule_updater.rule.upload.time").time():
            psc_s3_manager.upload_drec_aggregate_rule_set(
                rules, content_hash, created_at
            )

    metrics.health_metrics(1)
    if reporter is not None:
        asyncio.get_event_loop().run_until_complete(reporter.report_now())


def validate_domain(host_name):
    if not host_name or host_name == "UNKNOWN":
        raise AttributeError("DRE_C_HOST_NAME is not set")

    return host_name


if __name__ == "__main__":
    main()
