import logging
import time
from datetime import datetime
from urllib.parse import urlparse

from pscr_common.aws import get_boto3_client
import botocore

_logger = logging.getLogger(__name__)


def _download_s3_object(s3_client, s3_bucket, s3_key):
    return s3_client.get_object(Bucket=s3_bucket, Key=s3_key)


def _upload_s3_object(s3_client, s3_bucket, s3_key, body):
    s3_client.put_object(Body=body, Bucket=s3_bucket, Key=s3_key)


def _copy_s3_object(s3_client, s3_bucket, s3_source, s3_key):
    s3_client.copy_object(
        Bucket=s3_bucket, CopySource=f"{s3_bucket}/{s3_source}", Key=s3_key
    )


def _add_tags_to_s3_objects(s3_client, s3_bucket, s3_key, tag_obj):
    s3_client.put_object_tagging(Bucket=s3_bucket, Key=s3_key, Tagging=tag_obj)


def _get_s3_objects_tags(s3_client, s3_bucket, s3_key):
    return s3_client.get_object_tagging(Bucket=s3_bucket, Key=s3_key)


class RMSS3Manager:
    def __init__(self, s3_bucket_url, metrics, reporter):
        self.s3_bucket_url = s3_bucket_url
        self.s3_client = get_boto3_client("s3")
        self.metrics = metrics
        self.reporter = reporter

    def download_drec_aggregate_rule_set(self):
        """
        Function to Download Aggregated Rule Set from Pace Content S3 Bucket
        :return: aggregated rule set
        """
        try:
            s3_url_parsed = urlparse(self.s3_bucket_url)
            rules = _download_s3_object(
                self.s3_client, s3_url_parsed.netloc, s3_url_parsed.path[1:]
            )
            rule_size = rules.get("ContentLength") / 1024
            _logger.info(
                f"Successfully Downloaded Aggregated RuleSet with size {rule_size} KB "
                f"from S3 Bucket {s3_url_parsed.netloc}"
            )
            self.metrics.add_rule_size_metrics(rule_size)
            return rules["Body"].read()
        except botocore.exceptions.ClientError as error:
            self.metrics.add_error_metrics("rule.download")
            raise RuntimeError(
                f"Failed to Download Aggregated RuleSet due to Error {error.response['Error']['Code']}"
            ) from error


class PSCS3Manager:
    def __init__(self, config, metrics, reporter):
        self.s3_client = get_boto3_client("s3")
        self.config = config
        self.metrics = metrics
        self.reporter = reporter

    def upload_drec_aggregate_rule_set(self, rules, content_hash, created_at):
        """
         Function to upload aggregated Rule Set to Dre-C S3 bucket
        :param rules:
        :param content_hash:
        :param created_at:
        :return:
        """
        psc_s3_bucket = self.config.get("dre_c", "s3_bucket")
        psc_s3_bucket_dre_c_rules_key = self.config.get("dre_c", "s3_key")
        try:
            _upload_s3_object(
                self.s3_client, psc_s3_bucket, psc_s3_bucket_dre_c_rules_key, rules
            )
            _logger.info(
                f"Successfully Uploaded Aggregated RuleSet "
                f"to S3 Bucket {psc_s3_bucket}"
            )
            # Making thread sleep to print logs in order
            time.sleep(0.1)
            _logger.info("Dre-C Rule Updater Finished")
        except botocore.exceptions.ClientError as error:
            self.metrics.add_error_metrics("rule.upload")
            raise RuntimeError(
                f"Failed to Upload Aggregated RuleSet due to Error {error.response['Error']['Code']}"
            ) from error

        tag_obj = {
            "TagSet": [
                {
                    "Key": "content_hash",
                    "Value": content_hash,
                },
                {
                    "Key": "created_at",
                    "Value": created_at,
                },
            ],
        }
        try:
            _add_tags_to_s3_objects(
                self.s3_client, psc_s3_bucket, psc_s3_bucket_dre_c_rules_key, tag_obj
            )
        except botocore.exceptions.ClientError as error:
            self.metrics.add_error_metrics("rule.add_tags")
            _logger.warning(
                f"Failed to Add Tags to RuleSet due to Error {error.response['Error']['Code']}"
            )

    def validate_current_rules(self, content_hash):
        """
        Function to check if rule set is changed
        :param content_hash:
        :return:
        """
        try:
            psc_s3_bucket = self.config.get("dre_c", "s3_bucket")
            psc_s3_bucket_dre_c_rules_key = self.config.get("dre_c", "s3_key")
            response = _get_s3_objects_tags(
                self.s3_client, psc_s3_bucket, psc_s3_bucket_dre_c_rules_key
            )
        except botocore.exceptions.ClientError as error:
            self.metrics.add_error_metrics("rule.get_tags")
            _logger.warning(
                f"Failed to Get Tags due to Error {error.response['Error']['Code']}"
            )
            return False

        for tag in response["TagSet"]:
            if tag["Key"] == "content_hash" and tag["Value"] == content_hash:
                return True

        return False

    def create_backup(self):
        """
        Function to create back up of current rules before uploading new rules
        :return:
        """
        try:
            date_format = "%Y-%m-%d %H:%M:%S"
            tag_found = False
            psc_s3_bucket = self.config.get("dre_c", "s3_bucket")
            psc_s3_bucket_dre_c_rules_key = self.config.get("dre_c", "s3_key")
            psc_dre_c_rules_back_up_dir = self.config.get("dre_c", "rules_backup_dir")
            tag_response = _get_s3_objects_tags(
                self.s3_client, psc_s3_bucket, psc_s3_bucket_dre_c_rules_key
            )
            for tag in tag_response["TagSet"]:
                if tag["Key"] == "created_at":
                    try:
                        datetime_obj = datetime.fromtimestamp(
                            int(tag["Value"]) / 1000
                        ).strftime(date_format)
                        back_up_key = (
                            f"{psc_dre_c_rules_back_up_dir}/{datetime_obj}/rules.json"
                        )
                        _copy_s3_object(
                            self.s3_client,
                            psc_s3_bucket,
                            psc_s3_bucket_dre_c_rules_key,
                            back_up_key,
                        )
                        tag_found = True
                    except (OSError, ValueError):
                        self.metrics.add_error_metrics("s3_tag_format")
                        _logger.warning(
                            "Failed to parse created_at date on previous rule"
                        )

            if not tag_found:
                _logger.warning(
                    "created_at tag not found on previous rule,creating back up with Last Modified time"
                )
                response = self.s3_client.head_object(
                    Bucket=psc_s3_bucket, Key=psc_s3_bucket_dre_c_rules_key
                )
                back_up_key = (
                    f"{psc_dre_c_rules_back_up_dir}/rules_with_no_created_at_tag/"
                    f"{response['LastModified'].strftime(date_format)}/rules.json "
                )
                _copy_s3_object(
                    self.s3_client,
                    psc_s3_bucket,
                    psc_s3_bucket_dre_c_rules_key,
                    back_up_key,
                )

        except botocore.exceptions.ClientError as error:
            self.metrics.add_error_metrics("rule.backup")
            _logger.exception(
                f"Failed to create back up of previous rule set {error.response['Error']['Code']}"
            )
